@@ -1,6 +1,6 @@
 /* ws-dbrels.c
  *
- * Copyright (C) 2004 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,13 @@
  * USA
  */
 
+#include <libgnomedb-extra/libgnomedb-extra.h>
 #include "ws-dbrels.h"
 #include "workspace-page.h"
 #include <string.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
-#include <libgnomedb/libgnomedb.h>
+#include <glib/gi18n-lib.h>
 
 /* 
  * Main static functions 
@@ -34,12 +35,16 @@ static void ws_dbrels_dispose (GObject *object);
 static void ws_dbrels_finalize (GObject *object);
 
 /* WorkspacePage interface */
-static void         ws_dbrels_page_init       (WorkspacePageIface *iface);
-static gchar       *ws_dbrels_get_name        (WorkspacePage *iface);
-static gchar       *ws_dbrels_get_description (WorkspacePage *iface);
-static GtkWidget   *ws_dbrels_get_sel_button    (WorkspacePage *iface);
-static GtkWidget   *ws_dbrels_get_selector    (WorkspacePage *iface);
-static GtkWidget   *ws_dbrels_get_work_area   (WorkspacePage *iface);
+static void            ws_dbrels_page_init       (WorkspacePageIface *iface);
+static gchar          *ws_dbrels_get_name        (WorkspacePage *iface);
+static gchar          *ws_dbrels_get_description (WorkspacePage *iface);
+static GtkWidget      *ws_dbrels_get_sel_button  (WorkspacePage *iface);
+static GtkWidget      *ws_dbrels_get_selector    (WorkspacePage *iface);
+static GtkWidget      *ws_dbrels_get_work_area   (WorkspacePage *iface);
+static GtkActionGroup *ws_dbrels_get_actions     (WorkspacePage *iface);
+static const gchar    *ws_dbrels_get_actions_ui  (WorkspacePage *iface);
+
+static void            opened_dialog_closed_cb   (GtkWidget *dlg, WsDbrels *ws);
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
@@ -47,21 +52,20 @@ static GObjectClass  *parent_class = NULL;
 /* private structure */
 struct _WsDbrelsPrivate
 {
-	GnomeDbDict         *dict;
+	GdaDict        *dict;
 	GtkWidget      *selector;
 	GtkWidget      *work_area;
+	GtkActionGroup *actions;
 
 	GtkWidget      *notebook;
 	GtkWidget      *description;
 	GtkWidget      *canvas;
-	GtkWidget      *graph_add;
-	GtkWidget      *graph_edit;
-	GtkWidget      *graph_del;
 	GtkWidget      *zoom_in;
 	GtkWidget      *zoom_out;
 	GtkWidget      *zoom_fit;
 
 	GObject        *sel_obj;
+	GSList         *opened_dialogs;
 };
 
 GType
@@ -102,6 +106,8 @@ ws_dbrels_page_init (WorkspacePageIface *iface)
 	iface->get_sel_button = ws_dbrels_get_sel_button;
 	iface->get_selector = ws_dbrels_get_selector;
 	iface->get_work_area = ws_dbrels_get_work_area;
+	iface->get_actions = ws_dbrels_get_actions;
+	iface->get_actions_ui = ws_dbrels_get_actions_ui;
 }
 
 static void
@@ -125,19 +131,19 @@ static void ws_dbrels_initialize (WsDbrels *ws);
 
 /**
  * ws_dbrels_new
- * @dict: a #GnomeDbDict object
+ * @dict: a #GdaDict object
  *
  * Creates a new WsDbrels object
  *
  * Returns: the new object
  */
 GObject*
-ws_dbrels_new (GnomeDbDict *dict)
+ws_dbrels_new (GdaDict *dict)
 {
 	GObject  *obj;
 	WsDbrels *ws;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (dict && GDA_IS_DICT (dict), NULL);
 
 	obj = g_object_new (WS_DBRELS_TYPE, NULL);
 	ws = WS_DBRELS (obj);
@@ -160,6 +166,9 @@ ws_dbrels_dispose (GObject *object)
 
 	ws = WS_DBRELS (object);
 	if (ws->priv) {
+		while (ws->priv->opened_dialogs) 
+			gtk_widget_destroy (GTK_WIDGET (ws->priv->opened_dialogs->data));
+
 		if (ws->priv->selector) {
 			gtk_widget_destroy (ws->priv->selector);
 			ws->priv->selector = NULL;
@@ -170,6 +179,11 @@ ws_dbrels_dispose (GObject *object)
 			ws->priv->work_area = NULL;
 		}
 
+		if (ws->priv->actions) {
+			g_object_unref (ws->priv->actions);
+			ws->priv->actions = NULL;
+		}
+
 		if (ws->priv->dict) {
 			g_object_unref (G_OBJECT (ws->priv->dict));
 			ws->priv->dict = NULL;
@@ -204,9 +218,40 @@ static void selector_selection_changed_cb (GnomeDbSelector *mgsel, GObject *sel_
 static void zoom_in_clicked_cb (GtkButton *button, WsDbrels *ws);
 static void zoom_out_clicked_cb (GtkButton *button, WsDbrels *ws);
 static void zoom_fit_clicked_cb (GtkButton *button, WsDbrels *ws);
-static void graph_add_clicked_cb (GtkButton *button, WsDbrels *ws);
-static void graph_edit_clicked_cb (GtkButton *button, WsDbrels *ws);
-static void graph_del_clicked_cb (GtkButton *button, WsDbrels *ws);
+static void action_graph_add_cb (GtkButton *button, WsDbrels *ws);
+static void action_graph_edit_cb (GtkButton *button, WsDbrels *ws);
+static void action_graph_del_cb (GtkButton *button, WsDbrels *ws);
+
+static GtkActionEntry ui_actions[] = {
+	{ "Relation", NULL, "_Relation", NULL, "Relation", NULL },
+        { "GraphAdd", GTK_STOCK_ADD, "_New graph", NULL, "Create a new graph", 
+	  G_CALLBACK (action_graph_add_cb)},
+        { "GraphDel", GTK_STOCK_DELETE, "_Delete", NULL, "Delete the selected graph", 
+	  G_CALLBACK (action_graph_del_cb)},
+        { "GraphProp", GTK_STOCK_PROPERTIES, "_Properties", NULL, "Edit properties of the selected graph", 
+	  G_CALLBACK (action_graph_edit_cb)},
+};
+
+static const gchar *ui_actions_info =
+	"<ui>"
+	"  <menubar name='MenuBar'>"
+	"    <placeholder name='PageExtension'>"
+        "      <menu name='Relation' action='Relation'>"
+        "        <menuitem name='GraphAdd' action= 'GraphAdd'/>"
+        "        <menuitem name='GraphDel' action= 'GraphDel'/>"
+	"        <separator/>"
+	"        <menuitem name='GraphProp' action= 'GraphProp'/>"
+        "      </menu>"
+	"    </placeholder>"
+	"  </menubar>"
+        "  <toolbar  name='ToolBar'>"
+        "    <toolitem action='GraphAdd'/>"
+        "    <toolitem action='GraphDel'/>"
+        "    <separator/>"
+        "    <toolitem action='GraphProp'/>"
+        "  </toolbar>"
+        "</ui>";
+
 static void
 ws_dbrels_initialize (WsDbrels *ws)
 {
@@ -283,7 +328,7 @@ ws_dbrels_initialize (WsDbrels *ws)
 
 	bbox = gtk_hbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
-	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), bbox, TRUE, TRUE, 0);
 	gtk_widget_show (bbox);
 
 	button = gtk_button_new ();
@@ -328,35 +373,28 @@ ws_dbrels_initialize (WsDbrels *ws)
 	gtk_box_pack_start (GTK_BOX (hbox), bbox, FALSE, FALSE, 0);
 	gtk_widget_show (bbox);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
-	gtk_container_add (GTK_CONTAINER (bbox), button);
-	gtk_widget_show (button);
-	ws->priv->graph_add = button;
-	g_signal_connect (G_OBJECT (button), "clicked",
-			  G_CALLBACK (graph_add_clicked_cb), ws);
+	/* actions part */
+	ws->priv->actions = gtk_action_group_new ("Actions");
+	gtk_action_group_add_actions (ws->priv->actions, ui_actions, G_N_ELEMENTS (ui_actions), ws);
 
-	button = gtk_button_new_from_stock (GTK_STOCK_PROPERTIES);
-	gtk_container_add (GTK_CONTAINER (bbox), button);
-	gtk_widget_show (button);
-	gtk_widget_set_sensitive (button, FALSE);
-	ws->priv->graph_edit = button;
-	g_signal_connect (G_OBJECT (button), "clicked",
-			  G_CALLBACK (graph_edit_clicked_cb), ws);
-	
-	button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
-	gtk_container_add (GTK_CONTAINER (bbox), button);
-	gtk_widget_show (button);
-	gtk_widget_set_sensitive (button, FALSE);
-	ws->priv->graph_del = button;
-	g_signal_connect (G_OBJECT (button), "clicked",
-			  G_CALLBACK (graph_del_clicked_cb), ws);
+	if (ws->priv->actions) {
+		GtkAction *action;
+
+		action = gtk_action_group_get_action (ws->priv->actions, "GraphDel");
+		gtk_action_set_sensitive (action, FALSE);
+
+		action = gtk_action_group_get_action (ws->priv->actions, "GraphProp");
+		gtk_action_set_sensitive (action, FALSE);
+	}
 }
 
-static void graph_info_display_update (GnomeDbGraph *graph, WsDbrels *ws);
+static void graph_info_display_update (GdaGraph *graph, WsDbrels *ws);
 static void
 selector_selection_changed_cb (GnomeDbSelector *mgsel, GObject *sel_object, WsDbrels *ws)
 {
-	if (sel_object && !IS_GNOME_DB_GRAPH (sel_object))
+	GtkAction *action;
+
+	if (sel_object && !GDA_IS_GRAPH (sel_object))
 		return;
 	
 	ws->priv->sel_obj = sel_object;
@@ -366,9 +404,12 @@ selector_selection_changed_cb (GnomeDbSelector *mgsel, GObject *sel_object, WsDb
 	gtk_widget_set_sensitive (ws->priv->zoom_out, sel_object ? TRUE : FALSE);
 	gtk_widget_set_sensitive (ws->priv->zoom_fit, sel_object ? TRUE : FALSE);
 
-	gtk_widget_set_sensitive (ws->priv->graph_del, sel_object ? TRUE : FALSE);
-	gtk_widget_set_sensitive (ws->priv->graph_edit, sel_object ? TRUE : FALSE);
-	graph_info_display_update ((GnomeDbGraph*) sel_object, ws);
+	action = gtk_action_group_get_action (ws->priv->actions, "GraphDel");
+	gtk_action_set_sensitive (action, sel_object ? TRUE : FALSE);
+	action = gtk_action_group_get_action (ws->priv->actions, "GraphProp");
+	gtk_action_set_sensitive (action, sel_object ? TRUE : FALSE);
+
+	graph_info_display_update ((GdaGraph*) sel_object, ws);
 }
 
 static void
@@ -391,18 +432,18 @@ zoom_fit_clicked_cb (GtkButton *button, WsDbrels *ws)
 
 
 static void
-graph_add_clicked_cb (GtkButton *button, WsDbrels *ws)
+action_graph_add_cb (GtkButton *button, WsDbrels *ws)
 {
-	GnomeDbGraph *graph;
-	GnomeDbDict *dict = ws->priv->dict;
+	GdaGraph *graph;
+	GdaDict *dict = ws->priv->dict;
 
-	graph = GNOME_DB_GRAPH (gnome_db_graph_new (dict, GNOME_DB_GRAPH_DB_RELATIONS));
-	gnome_db_dict_assume_graph (dict, graph);
+	graph = GDA_GRAPH (gda_graph_new (dict, GDA_GRAPH_DB_RELATIONS));
+	gda_dict_assume_object (dict, (GdaObject *) graph);
 	g_object_unref (G_OBJECT (graph));
 }
 
 static void
-graph_edit_clicked_cb (GtkButton *button, WsDbrels *ws)
+action_graph_edit_cb (GtkButton *button, WsDbrels *ws)
 {
 	const gchar *str;
 	GtkWidget *wid, *vbox, *hbox, *dlg, *table, *name, *descr;
@@ -447,14 +488,14 @@ graph_edit_clicked_cb (GtkButton *button, WsDbrels *ws)
 	name = gtk_entry_new ();
 	gtk_widget_show (name);
 	gtk_table_attach (GTK_TABLE (table), name, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-	str = gnome_db_base_get_name (GNOME_DB_BASE (sel_graph));
+	str = gda_object_get_name (GDA_OBJECT (sel_graph));
 	if (str)
 		gtk_entry_set_text (GTK_ENTRY (name), str);
 
 	descr = gtk_entry_new ();
 	gtk_widget_show (descr);
 	gtk_table_attach (GTK_TABLE (table), descr, 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
-	str = gnome_db_base_get_description (GNOME_DB_BASE (sel_graph));
+	str = gda_object_get_description (GDA_OBJECT (sel_graph));
 	if (str)
 		gtk_entry_set_text (GTK_ENTRY (descr), str);
 
@@ -469,40 +510,46 @@ graph_edit_clicked_cb (GtkButton *button, WsDbrels *ws)
 	result = gtk_dialog_run (GTK_DIALOG (dlg));
 	if (result == GTK_RESPONSE_ACCEPT) {
 		str = gtk_entry_get_text (GTK_ENTRY (name));
-		gnome_db_base_set_name (GNOME_DB_BASE (sel_graph), str);
+		gda_object_set_name (GDA_OBJECT (sel_graph), str);
 		str = gtk_entry_get_text (GTK_ENTRY (descr));
-		gnome_db_base_set_description (GNOME_DB_BASE (sel_graph), str);
+		gda_object_set_description (GDA_OBJECT (sel_graph), str);
 		
 	}
 	gtk_object_destroy (GTK_OBJECT (dlg));
 }
 
 static void
-graph_del_clicked_cb (GtkButton *button, WsDbrels *ws)
+action_graph_del_cb (GtkButton *button, WsDbrels *ws)
 {
 	GObject *sel_graph = ws->priv->sel_obj;
 	GtkWidget *dlg;
-	
+	const gchar *name;
+
 	if (!sel_graph)
 		return;
 
-	dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-				      _("Do you want to delete the '%s' graph?"), 
-				      gnome_db_base_get_name (GNOME_DB_BASE (sel_graph)));
+	name = gda_object_get_name (GDA_OBJECT (sel_graph));
+	if (name && *name)
+		dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+					      _("Do you want to delete the '%s' graph?"), name);
+	else
+		dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+					      _("Do you want to delete the graph <no name>?"));
+	
 	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) 
-		gnome_db_base_nullify (GNOME_DB_BASE (sel_graph));
+		gda_object_destroy (GDA_OBJECT (sel_graph));
 
 	gtk_widget_destroy (dlg);
 }
 
 static void
-graph_info_display_update (GnomeDbGraph *graph, WsDbrels *ws)
+graph_info_display_update (GdaGraph *graph, WsDbrels *ws)
 {
 	const gchar *str = NULL;
 	gchar *title = NULL;
 
 	if (graph)
-		str = gnome_db_base_get_description (GNOME_DB_BASE (graph));
+		str = gda_object_get_description (GDA_OBJECT (graph));
 	if (str && *str) 
 		gtk_label_set_text (GTK_LABEL (ws->priv->description), str);
 	else
@@ -526,7 +573,7 @@ graph_info_display_update (GnomeDbGraph *graph, WsDbrels *ws)
 static gchar *
 ws_dbrels_get_name (WorkspacePage *iface)
 {
-	g_return_val_if_fail (iface && IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
 	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
 
 	return g_strdup_printf (_("Database Relations"));
@@ -538,13 +585,13 @@ ws_dbrels_get_description (WorkspacePage *iface)
 	gchar *str = NULL;
 	WsDbrels *ws;
 
-	g_return_val_if_fail (iface && IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
 	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
 
 	ws = WS_DBRELS (iface);
 	if (ws->priv->sel_obj) {
 		const gchar *cstr;
-		 cstr =  gnome_db_base_get_name (GNOME_DB_BASE (ws->priv->sel_obj));
+		 cstr =  gda_object_get_name (GDA_OBJECT (ws->priv->sel_obj));
 		 str = g_strdup_printf ("Database relation: <b>%s</b>", (cstr && *cstr) ? cstr : "---");
 	}
 	else
@@ -559,7 +606,7 @@ ws_dbrels_get_sel_button (WorkspacePage *iface)
 	GdkPixbuf *pixbuf;
 	GtkWidget *button, *wid, *hbox;
 
-	g_return_val_if_fail (iface && IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
 	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
 
 	hbox = gtk_hbox_new (FALSE, 0);
@@ -584,7 +631,7 @@ ws_dbrels_get_sel_button (WorkspacePage *iface)
 static GtkWidget *
 ws_dbrels_get_selector (WorkspacePage *iface)
 {
-	g_return_val_if_fail (iface && IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
 	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
 
 	return WS_DBRELS (iface)->priv->selector;
@@ -593,8 +640,32 @@ ws_dbrels_get_selector (WorkspacePage *iface)
 static GtkWidget *
 ws_dbrels_get_work_area (WorkspacePage *iface)
 {
-	g_return_val_if_fail (iface && IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
 	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
 
 	return WS_DBRELS (iface)->priv->work_area;
 }
+
+static GtkActionGroup *
+ws_dbrels_get_actions (WorkspacePage *iface)
+{
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
+
+	return WS_DBRELS (iface)->priv->actions;
+}
+
+static const gchar *
+ws_dbrels_get_actions_ui (WorkspacePage *iface)
+{
+	g_return_val_if_fail (IS_WS_DBRELS (iface), NULL);
+	g_return_val_if_fail (WS_DBRELS (iface)->priv, NULL);
+
+	return ui_actions_info;
+}
+
+static void
+opened_dialog_closed_cb (GtkWidget *dlg, WsDbrels *ws)
+{
+	ws->priv->opened_dialogs = g_slist_remove (ws->priv->opened_dialogs, dlg);
+}
